// -*- C++ -*-
/*!
 * @file  iMCs01.cpp
 * @brief iXs Research iMCs01 Component
 * @date $Date$
 *
 * @author ysuga [at] ysuga.net
 * url: http://www.ysuga.net
 *
 * GPL
 *
 * $Id$
 */

#include "iMCs01.h"

using namespace ysuga_net;

// Module specification
// <rtc-template block="module_spec">
static const char* imcs01_spec[] =
  {
    "implementation_id", "iMCs01",
    "type_name",         "iMCs01",
    "description",       "iXs Research iMCs01 Component",
    "version",           "1.0.0",
    "vendor",            "ysuga.net",
    "category",          "Actuator",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "16",
    "language",          "C++",
    "lang_type",         "compile",
    //"exec_cxt.periodic.rate", "100.0",
    // Configuration variables
    "conf.default.board_no", "0",
    "conf.default.kp0", DEFAULT_KP,
    "conf.default.kpx0", DEFAULT_KPX,
    "conf.default.kd0", DEFAULT_KD,
    "conf.default.kdx0", DEFAULT_KDX,
    "conf.default.ki0", DEFAULT_KI,
    "conf.default.kix0", DEFAULT_KIX,
    "conf.default.kp1", DEFAULT_KP,
    "conf.default.kpx1", DEFAULT_KPX,
    "conf.default.kd1", DEFAULT_KD,
    "conf.default.kdx1", DEFAULT_KDX,
    "conf.default.ki1", DEFAULT_KI,
    "conf.default.kix1", DEFAULT_KIX,
	"conf.default.kp2", DEFAULT_KP,
    "conf.default.kpx2", DEFAULT_KPX,
    "conf.default.kd2", DEFAULT_KD,
    "conf.default.kdx2", DEFAULT_KDX,
    "conf.default.ki2", DEFAULT_KI,
    "conf.default.kix2", DEFAULT_KIX,
	"conf.default.kp3", DEFAULT_KP,
    "conf.default.kpx3", DEFAULT_KPX,
    "conf.default.kd3", DEFAULT_KD,
    "conf.default.kdx3", DEFAULT_KDX,
    "conf.default.ki3", DEFAULT_KI,
    "conf.default.kix3", DEFAULT_KIX,
	"conf.default.driver0", "imds03",
    "conf.default.driver1", "imds03",
    "conf.default.driver2", "imds03",
    "conf.default.driver3", "imds03",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
iMCs01::iMCs01(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    targetIn("targetCount", target),
    currentOut("currentCount", current),
    voltageOut("currentVoltage", voltage)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
iMCs01::~iMCs01()
{
}



RTC::ReturnCode_t iMCs01::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("targetCount", targetIn);
  
  // Set OutPort buffer
  registerOutPort("currentCount", currentOut);
  registerOutPort("currentVoltage", voltageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("board_no", board_no, "0");
  bindParameter("kp0", kp0, DEFAULT_KP);
  bindParameter("kpx0", kpx0, DEFAULT_KPX);
  bindParameter("kd0", kd0, DEFAULT_KD);
  bindParameter("kdx0", kdx0, DEFAULT_KDX);
  bindParameter("ki0", ki0, DEFAULT_KI);
  bindParameter("kix0", kix0, DEFAULT_KIX);
  bindParameter("kp1", kp1, DEFAULT_KP);
  bindParameter("kpx1", kpx1, DEFAULT_KPX);
  bindParameter("kd1", kd1, DEFAULT_KD);
  bindParameter("kdx1", kdx1, DEFAULT_KDX);
  bindParameter("ki1", ki1, DEFAULT_KI);
  bindParameter("kix1", kix1, DEFAULT_KIX);
  bindParameter("kp2", kp2, DEFAULT_KP);
  bindParameter("kpx2", kpx2, DEFAULT_KPX);
  bindParameter("kd2", kd2, DEFAULT_KD);
  bindParameter("kdx2", kdx2, DEFAULT_KDX);
  bindParameter("ki2", ki2, DEFAULT_KI);
  bindParameter("kix2", kix2, DEFAULT_KIX);
  bindParameter("kp3", kp3, DEFAULT_KP);
  bindParameter("kpx3", kpx3, DEFAULT_KPX);
  bindParameter("kd3", kd3, DEFAULT_KD);
  bindParameter("kdx3", kdx3, DEFAULT_KDX);
  bindParameter("ki3", ki3, DEFAULT_KI);
  bindParameter("kix3", kix3, DEFAULT_KIX);
  bindParameter("driver0", driver0, "imds03");
  bindParameter("driver1", driver1, "imds03");
  bindParameter("driver2", driver2, "imds03");
  bindParameter("driver3", driver3, "imds03");
  
  // </rtc-template>
 	current.data.length(4);
	voltage.data.length(4);
	
	return RTC::RTC_OK;
}


RTC::ReturnCode_t iMCs01::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t iMCs01::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t iMCs01::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "Activating iMCs01" << std::endl;
	try {
		m_piMCs01Device = new iMCs01Device(board_no);
	} catch (std::exception& e) {
		std::cout << "Exception occurred in iMCs01::onActivated()" << std::endl;
		std::cout << " -- board_no: " << board_no << std::endl;
		std::cout << " -- message : " << e.what() << std::endl;
		return RTC::RTC_ERROR;
	}
  return RTC::RTC_OK;
}


RTC::ReturnCode_t iMCs01::onDeactivated(RTC::UniqueId ec_id)
{
	try {
		delete m_piMCs01Device;
	} catch (std::exception& e) {
		std::cout << "Exception occurred in iMCs01::onDeactivated()" << std::endl;
		std::cout << " -- board_no: " << board_no << std::endl;
		std::cout << " -- message : " << e.what() << std::endl;
		return RTC::RTC_ERROR;
	} 
  return RTC::RTC_OK;
}


RTC::ReturnCode_t iMCs01::onExecute(RTC::UniqueId ec_id)
{
	if(targetIn.isNew()) {
		targetIn.read();
		if(target.data.length() == 4) {
			m_piMCs01Device->SetGain(0, kp0, kpx0, kd0, kdx0, ki0, kix0);
			m_piMCs01Device->SetGain(1, kp1, kpx1, kd1, kdx1, ki1, kix1);
			m_piMCs01Device->SetGain(2, kp2, kpx2, kd2, kdx2, ki2, kix2);
			m_piMCs01Device->SetGain(3, kp3, kpx3, kd3, kdx3, ki3, kix3);
			for(int i = 0;i < 4;i++) {
				m_piMCs01Device->SetTargetCount(i, target.data[i]);
			}
			try {
				m_piMCs01Device->WriteCommandPacket();
			} catch (std::exception &e) {
				std::cout << "Excepiton occured in writing command packet:" << e.what() << std::endl;
				return RTC::RTC_ERROR;
			}
		}
	}


	try {
		m_piMCs01Device->ReadStatusPacket();
	} catch (std::exception &e) {
		std::cout << "Excepiton occured in reading status packet:" << e.what() << std::endl;
		return RTC::RTC_ERROR;
	}


	for(int i = 0;i < 4;i++) {
		current.data[i] = m_piMCs01Device->GetCount(i);
		voltage.data[i] = (double)m_piMCs01Device->GetAD(i) / 1024 * 5.0;
	}
	currentOut.write();
	voltageOut.write();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t iMCs01::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t iMCs01::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t iMCs01::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  DLL_EXPORT void iMCs01Init(RTC::Manager* manager)
  {
    coil::Properties profile(imcs01_spec);
    manager->registerFactory(profile,
                             RTC::Create<iMCs01>,
                             RTC::Delete<iMCs01>);
  }
  
};


