// -*- C++ -*-
/*!
 * @file  iMCs01.h
 * @brief iXs Research iMCs01 Component
 * @date  $Date$
 *
 * @author ysuga [at] ysuga.net
 * url: http://www.ysuga.net
 *
 * GPL
 *
 * $Id$
 */

#ifndef IMCS01_H
#define IMCS01_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>


#include "iMCs01Device.h"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

/*!
 * @class iMCs01
 * @brief iXs Research iMCs01 Component
 *
 * iMCs01 is a product by iXs Research. http://www.ixs.co.jp
 *
 * proportional gain kp is defined by Kp / Kpx.
 * differential gain kd is kd / kdx
 * integral gain ki is ki / kix.
 *
 * http://www.ixs.co.jp
 * http://www.ysuga.net
 *
 */
class iMCs01
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  iMCs01(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~iMCs01();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 
   * - Name:  board_no
   * - DefaultValue: 0
   */
  int board_no;
  /*!
   * 
   * - Name:  kp0
   * - DefaultValue: 10
   */
  short kp0;
  /*!
   * 
   * - Name:  kpx0
   * - DefaultValue: 100
   */
  unsigned short kpx0;
  /*!
   * 
   * - Name:  kd0
   * - DefaultValue: 10
   */
  short kd0;
  /*!
   * 
   * - Name:  kdx0
   * - DefaultValue: 100
   */
  unsigned short kdx0;
  /*!
   * 
   * - Name:  ki0
   * - DefaultValue: 0
   */
  short ki0;
  /*!
   * 
   * - Name:  kix0
   * - DefaultValue: 100
   */
  unsigned short kix0;
  /*!
   * 
   * - Name:  kp1
   * - DefaultValue: 10
   */
  short kp1;
  /*!
   * 
   * - Name:  kpx1
   * - DefaultValue: 100
   */
  unsigned short kpx1;
  /*!
   * 
   * - Name:  kd1
   * - DefaultValue: 10
   */
  short kd1;
  /*!
   * 
   * - Name:  kdx1
   * - DefaultValue: 100
   */
  unsigned short kdx1;
  /*!
   * 
   * - Name:  ki1
   * - DefaultValue: 0
   */
  short ki1;
  /*!
   * 
   * - Name:  kix1
   * - DefaultValue: 100
   */
  unsigned short kix1;
  /*!
   * 
   * - Name:  kp2
   * - DefaultValue: 10
   */
  short kp2;
  /*!
   * 
   * - Name:  kpx2
   * - DefaultValue: 100
   */
  unsigned short kpx2;
  /*!
   * 
   * - Name:  kd2
   * - DefaultValue: 10
   */
  short kd2;
  /*!
   * 
   * - Name:  kdx2
   * - DefaultValue: 100
   */
  unsigned short kdx2;
  /*!
   * 
   * - Name:  ki2
   * - DefaultValue: 0
   */
  short ki2;
  /*!
   * 
   * - Name:  kix2
   * - DefaultValue: 100
   */
  unsigned short kix2;
  /*!
   * 
   * - Name:  kp3
   * - DefaultValue: 10
   */
  short kp3;
  /*!
   * 
   * - Name:  kpx3
   * - DefaultValue: 100
   */
  unsigned short kpx3;
  /*!
   * 
   * - Name:  kd3
   * - DefaultValue: 10
   */
  short kd3;
  /*!
   * 
   * - Name:  kdx3
   * - DefaultValue: 100
   */
  unsigned short kdx3;
  /*!
   * 
   * - Name:  ki3
   * - DefaultValue: 0
   */
  short ki3;
  /*!
   * 
   * - Name:  kix3
   * - DefaultValue: 100
   */
  unsigned short kix3;
  /*!
   * 
   * - Name:  driver0
   * - DefaultValue: imds03
   */
  std::string driver0;
  /*!
   * 
   * - Name:  driver1
   * - DefaultValue: imds03
   */
  std::string driver1;
  /*!
   * 
   * - Name:  driver2
   * - DefaultValue: imds03
   */
  std::string driver2;
  /*!
   * 
   * - Name:  driver3
   * - DefaultValue: imds03
   */
  std::string driver3;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedShortSeq target;
  /*!
   */
  InPort<TimedShortSeq> targetIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedShortSeq current;
  /*!
   */
  OutPort<TimedShortSeq> currentOut;
  TimedDoubleSeq voltage;
  /*!
   */
  OutPort<TimedDoubleSeq> voltageOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

	 ysuga_net::iMCs01Device* m_piMCs01Device;

};

#define DEFAULT_KP  "10"
#define DEFAULT_KPX "100"

#define DEFAULT_KD  "10"
#define DEFAULT_KDX "100"

#define DEFAULT_KI  "0"
#define DEFAULT_KIX "100"

extern "C"
{
  DLL_EXPORT void iMCs01Init(RTC::Manager* manager);
};

#endif // IMCS01_H
