#include "uusbd.h"
#include "iMCs01Device.h"


using namespace ysuga_net;

/**
 * Constructor
 */
iMCs01Device::iMCs01Device(int device)
{
#ifdef WIN32
	Sleep(1000 * device);
  DWORD ret;
  m_hUSB = Uusbd_Open_mask(UU_MASK_VENDOR | UU_MASK_PRODUCT, 0, 0, 
			   0xFF8, 0x001, 0);
  if(m_hUSB == INVALID_HANDLE_VALUE) {
    throw iMCs01OpenException();
  } else {
  }
  
  ret = Uusbd_Check(m_hUSB);
  switch(ret){
  case UU_CHECK_OK:
    break;
  case UU_CHECK_NOTOPEN:
    throw iMCs01OpenException();
    break;
  case UU_CHECK_NODEVICE:
    throw iMCs01OpenException();
    break;
  }

  m_MaxPacketLength = GetMaxPacket(0, 0);
  if(m_MaxPacketLength == 0){
    throw iMCs01OpenException();
  }

  
  m_hStatus = Uusbd_OpenPipe(m_hUSB, 0, 2);
  if(m_hCommand == INVALID_HANDLE_VALUE)
    throw iMCs01OpenException();
  m_hCommand = Uusbd_OpenPipe(m_hUSB, 0, 1);
  if(m_hCommand == INVALID_HANDLE_VALUE)
    throw iMCs01OpenException();
  m_hControl = Uusbd_OpenPipe(m_hUSB, 0, 3);
  if(m_hControl == INVALID_HANDLE_VALUE)
    throw iMCs01OpenException();

#else // Linux


#endif


	Reset();
}


/*
 ~iMCs01Device
 */
iMCs01Device::~iMCs01Device(void)
{
  for(char i=0; i<4; i++){
    this->SetGain(i, 0, 100, 0, 100, 0, 100);
  }
  WriteCommandPacket();

  if(IsValidFileDescriptor()) {
#ifdef WIN32
    CloseHandle(m_hStatus);
    CloseHandle(m_hCommand);
    CloseHandle(m_hControl);
    Uusbd_Close(m_hUSB);
#else // Linux

#endif
  }
}


void iMCs01Device::WriteControlPacket()
{

#ifdef WIN32
  DWORD size;
  if(WriteFile(m_hControl, &m_Control , sizeof(m_Control), &size, NULL)<0){
    throw iMCs01CommunicationException();
  }
  Uusbd_ResetPipe(m_hControl);
#else // Linux

#endif
}

void iMCs01Device::WriteCommandPacket() {
  #ifdef WIN32
	DWORD size;
  if(WriteFile(m_hCommand, &m_Command, sizeof(m_Command), &size, NULL)<0){
    throw iMCs01CommunicationException();
  }
  Uusbd_ResetPipe(m_hCommand);
#else // Linux

#endif
}

void iMCs01Device::ReadStatusPacket() {
#ifdef WIN32
	DWORD size, ret;

  if(ReadFile(m_hStatus, &m_Status, sizeof(m_Status), &size, NULL) <= 0){
    ret = Uusbd_Check(m_hUSB);
    if(ret != UU_CHECK_OK) {
      throw iMCs01CommunicationException();
    }
  }
  Uusbd_ResetPipe(m_hStatus);
#else // Linux

#endif
}

/**
 * Resetting iMCs01
 */
void iMCs01Device::Reset(void)
{
  m_Control.retval      = 0;
  m_Control.setoffset   = CH0 | CH1 | CH2 | CH3;
  m_Control.setcounter  = CH0 | CH1 | CH2 | CH3;
  m_Control.resetint    = CH0 | CH1 | CH2 | CH3;
  m_Control.selin       = /* CH0 | CH1 | CH2 | CH3 | */SET_SELECT;
  m_Control.dout        = 0;
  m_Control.selout      = SET_SELECT | CH0 | CH1 | CH2 | CH3;
  m_Control.offset[0]   = m_Control.offset[1] = m_Control.offset[2] = m_Control.offset[3] = 0x7fff;
  m_Control.counter[0]  = m_Control.counter[1] = m_Control.counter[2] = m_Control.counter[3] = 0;
  m_Control.posneg      = SET_POSNEG | CH0 | CH1 | CH2 | CH3;
  m_Control.breaks      = SET_BREAKS | CH0 | CH1 | CH2 | CH3;
  m_Control.magicno     = 0x00;
  m_Control.wrrom       = 0;
 
  WriteControlPacket();

  m_Control.setoffset   = 0;
  m_Control.setcounter  = 0;
  m_Control.resetint    = 0;
	

  for (int i = 0;i < 4;i++){
    m_Command.ch[i].x   = 0;
    m_Command.ch[i].d   = 0;
    m_Command.ch[i].kp  = 0;
    m_Command.ch[i].kpx = 100;
    m_Command.ch[i].kd  = 0;
    m_Command.ch[i].kdx = 100;
    m_Command.ch[i].ki  = 0;
    m_Command.ch[i].kix = 100;
  }

  WriteCommandPacket();
}


/*
 * GetMaxPacket
 */
unsigned short iMCs01Device::GetMaxPacket(unsigned long IfNum, unsigned long PipeNum)
{
#ifdef WIN32
  char *buf, *p;
  USHORT MaxLen;
  DWORD PipeCount , len;
  BOOL ok;
  USB_CONFIGURATION_DESCRIPTOR conf;
  PUSB_ENDPOINT_DESCRIPTOR EndpDescriptor;
  
  ok = Uusbd_GetConfigurationDescriptor(m_hUSB,(char*)&conf,sizeof(conf));
  if(!ok) return 0;
  len = conf.wTotalLength;
  buf = (char*)malloc(len);
  
   ok = Uusbd_GetConfigurationDescriptor(m_hUSB , buf , len);
  if(!ok) return 0;
  
  p = buf; MaxLen = 0; PipeCount = 0; ok = FALSE;
  while(len>0){
    if(p[1] == USB_INTERFACE_DESCRIPTOR_TYPE){
      if((unsigned) p[2] == IfNum) ok = TRUE;
      else ok =FALSE;
      PipeCount = 0;
    }
    if(p[1] == USB_ENDPOINT_DESCRIPTOR_TYPE && ok){
      if(PipeCount == PipeNum){
	EndpDescriptor =(PUSB_ENDPOINT_DESCRIPTOR)p;
	MaxLen = EndpDescriptor->wMaxPacketSize;
	break;
      }
      PipeCount++;
    }
    p += p[0];
    len -= p[0];
  }
  free(buf);
  
  return MaxLen;
#else // Linux
  return 0;
#endif
}
