/**
 * iMCs01Device class
 * author : ysuga ( ysuga@ysuga.net )
 */

#ifndef IMCS01DEVICE_HEADER_INCLUDED
#define IMCS01DEVICE_HEADER_INCLUDED

#include <exception>
#include "uusbd.h"
#include "urobotc.h"

#ifdef WIN32
#ifdef _USRDLL
#define DLLEXPORT __declspec(dllexport)
#else
#define DLLEXPORT __declspec(dllimport)
#endif
#else // Linux
#define DLLEXPORT
#endif

namespace ysuga_net {

  class DLLEXPORT iMCs01OpenException : public std::exception {
  public:
	iMCs01OpenException():std::exception() {
    }
    
    virtual const char *what() {
      return "Opening iMCs01 failed.";
    }
  };

  class DLLEXPORT iMCs01CommunicationException : public std::exception {
  public:
	  iMCs01CommunicationException() : std::exception() {
    }
    
   virtual const char *what() {
      return "Communicating iMCs01 failed.";
    }

  };

  class DLLEXPORT iMCs01InvalidChannelException : public std::exception {
  public:
	  iMCs01InvalidChannelException() : std::exception() {
    }
    
    virtual const char *what() {
      return "Setting Invalid Channel.";
    }

  };


  /**
   * iMCs01Device
   *
   **/
  class DLLEXPORT iMCs01Device
  {
  private:
#ifdef WIN32
    HUSB m_hUSB;
    DWORD m_MaxPacketLength;
    HANDLE m_hStatus, m_hControl, m_hCommand;
#else // Linux
    int m_Fd;
#endif
  public:
    int IsValidFileDescriptor() {
#ifdef WIN32
      return (m_hUSB != INVALID_HANDLE_VALUE);
#else
      return (m_Fd != 0);
#endif
    }

      private:
    struct ccmd m_Control;
    struct uout m_Command;
    struct uin  m_Status;
    
  public:
    /**
     * Constructor
     */
    iMCs01Device(int device);

    /**
     * Destructor
     */
    virtual ~iMCs01Device(void);
  private:
    unsigned short GetMaxPacket(unsigned long IfNum, unsigned long PipeNum);



    void ChannelCheck(int channel) {
      if(channel < 0 || channel >= 4) {
	throw iMCs01InvalidChannelException();
      }
    }

  public:
	void WriteCommandPacket();
    void WriteControlPacket();
    void ReadStatusPacket();
  public:
    void Reset(void);
    void Read(void) {ReadStatusPacket();}
    void Write(void) {WriteCommandPacket();}

  public:    
    unsigned short GetAD(int channel) {
      ChannelCheck(channel);
      return (m_Status.ad[channel] >> 5);
    }

    void SetTargetCount(int channel, short count) {
      ChannelCheck(channel);
      m_Command.ch[channel].x = count;
    }

    short GetCount(int channel) {
      ChannelCheck(channel);
      return m_Status.ct[channel];
    }

    void SetGain(int channel,
		 short Kp, unsigned short Kpx,
		 short Kd, unsigned short Kdx,
		 short Ki, unsigned short Kix) {
      ChannelCheck(channel);
      m_Command.ch[channel].kp = Kp; m_Command.ch[channel].kpx = Kpx;
      m_Command.ch[channel].kd = Kd; m_Command.ch[channel].kdx = Kdx;
      m_Command.ch[channel].ki = Ki; m_Command.ch[channel].kix = Kix;
    }

    void SetCounter(int channel, short count) {
      ChannelCheck(channel);
	  m_Control.setoffset = 0;
	  m_Control.resetint  = 0;
	  m_Control.setcounter = 0x01 << channel;
      m_Control.counter[channel] = count;
	  WriteControlPacket();
    }
    

  };
  
};

#endif
