// -*- C++ -*-
/*!
 * @file  iMCs01Test.cpp
 * @brief iMCs01 Test Component
 * @date $Date$
 *
 * $Id$
 */

#include "iMCs01Test.h"

// Module specification
// <rtc-template block="module_spec">
static const char* imcs01test_spec[] =
  {
    "implementation_id", "iMCs01Test",
    "type_name",         "iMCs01Test",
    "description",       "iMCs01 Test Component",
    "version",           "1.0.0",
    "vendor",            "ysuga.net",
    "category",          "Test",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "100.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
iMCs01Test::iMCs01Test(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    currentIn("current", current),
    voltageIn("voltage", voltage),
    targetOut("target", target)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
iMCs01Test::~iMCs01Test()
{
}



RTC::ReturnCode_t iMCs01Test::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("current", currentIn);
  registerInPort("voltage", voltageIn);
  
  // Set OutPort buffer
  registerOutPort("target", targetOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t iMCs01Test::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01Test::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01Test::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t iMCs01Test::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

static short g_current;
static double g_voltage;

RTC::ReturnCode_t iMCs01Test::onDeactivated(RTC::UniqueId ec_id)
{

  return RTC::RTC_OK;
}

int step = 100;
RTC::ReturnCode_t iMCs01Test::onExecute(RTC::UniqueId ec_id)
{
	if(currentIn.isNew()) {
		currentIn.read();
		g_current = this->current.data[3];
	}

	if(voltageIn.isNew()) {
		voltageIn.read();
		g_voltage = this->voltage.data[3];
	}

	char c; int buf;
	static short vtarget;

	std::cout << "Input command: (h:help):" <<std::ends;
	
	std::cin >> c;
	switch(c) {
		case 'h':
			std::cout << "v: view\n"
				"p:positive move\n"
				"n:negative move\n"
				"s:change step(" << step << ")\n"  << std::endl;
			break;
		case 's':
			std::cout << "input step:" << std::ends;
			std::cin >> buf;
			std::cout << "new step is " <<   buf << ". OK?" << std::ends;
			std::cin >> c;
			if(c == 'y') {
				std::cout << "updated." << std::endl;
				step = buf;
			}
			else std::cout << "aborted." << std::endl;
			break;
		case 'p':
			vtarget += step;
			target.data.length(4);
			target.data[3] = vtarget;
			targetOut.write();
			break;
		case 'n':
			vtarget -= step;
			target.data.length(4);
			target.data[3] = vtarget;
			targetOut.write();
			break;
		case 'v':
			std::cout << "C:" << g_current << ", A:" << g_voltage << std::endl;
			break;
		default:
			break;
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t iMCs01Test::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01Test::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01Test::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01Test::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t iMCs01Test::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void iMCs01TestInit(RTC::Manager* manager)
  {
    coil::Properties profile(imcs01test_spec);
    manager->registerFactory(profile,
                             RTC::Create<iMCs01Test>,
                             RTC::Delete<iMCs01Test>);
  }
  
};


